@@ -12,7 +12,6 @@
 
 #include <KCalendarCore/Incidence>
 #include <KCheckableProxyModel>
-#include <KFormat>
 #include <QAbstractItemModelTester>
 #include <QSignalSpy>
 #include <QTest>
@@ -24,7 +23,7 @@ class TodoSortFilterProxyModelTest : public QObject
 
 public:
     TodoSortFilterProxyModelTest() = default;
-    ~TodoSortFilterProxyModelTest() = default;
+    ~TodoSortFilterProxyModelTest() override = default;
 
     void checkAllItems(KCheckableProxyModel *model, const QModelIndex &parent = QModelIndex())
     {
