@@ -12,9 +12,12 @@
 
 // Akonadi
 #include "kalendar_debug.h"
+
+#include <Akonadi/AgentFilterProxyModel>
 #include <Akonadi/AgentInstanceModel>
 #include <Akonadi/AgentManager>
 #include <Akonadi/AttributeFactory>
+#include <Akonadi/Collection>
 #include <Akonadi/CollectionColorAttribute>
 #include <Akonadi/CollectionDeleteJob>
 #include <Akonadi/CollectionIdentificationAttribute>
@@ -22,31 +25,38 @@
 #include <Akonadi/CollectionPropertiesDialog>
 #include <Akonadi/CollectionUtils>
 #include <Akonadi/Control>
+#include <Akonadi/ETMViewStateSaver>
 #include <Akonadi/EntityDisplayAttribute>
+#include <Akonadi/EntityRightsFilterModel>
 #include <Akonadi/EntityTreeModel>
 #include <Akonadi/ItemModifyJob>
 #include <Akonadi/ItemMoveJob>
 #include <Akonadi/Monitor>
-#include <akonadi-calendar_version.h>
+#include <akonadi_version.h>
 #if AKONADICALENDAR_VERSION > QT_VERSION_CHECK(5, 19, 41)
 #include <Akonadi/History>
 #else
 #include <Akonadi/Calendar/History>
 #endif
+#include <CalendarSupport/KCalPrefs>
+#include <CalendarSupport/Utils>
 #include <EventViews/Prefs>
 #include <KCheckableProxyModel>
+#include <KDescendantsProxyModel>
 #include <KLocalizedString>
 #include <QApplication>
-#include <QDebug>
 #include <QPointer>
-#include <QRandomGenerator>
+#include <models/todosortfilterproxymodel.h>
 
 #if AKONADICALENDAR_VERSION > QT_VERSION_CHECK(5, 19, 41)
-#include <Akonadi/ETMCalendar>
 #else
 #include <etmcalendar.h>
 #endif
 
+#include <colorproxymodel.h>
+#include <incidencewrapper.h>
+#include <sortedcollectionproxymodel.h>
+
 using namespace Akonadi;
 
 static Akonadi::EntityTreeModel *findEtm(QAbstractItemModel *model)
@@ -68,24 +78,6 @@ bool isStandardCalendar(Akonadi::Collection::Id id)
     return id == CalendarSupport::KCalPrefs::instance()->defaultCalendarId();
 }
 
-static bool hasCompatibleMimeTypes(const Akonadi::Collection &collection)
-{
-    static QStringList goodMimeTypes;
-
-    if (goodMimeTypes.isEmpty()) {
-        goodMimeTypes << QStringLiteral("text/calendar") << KCalendarCore::Event::eventMimeType() << KCalendarCore::Todo::todoMimeType()
-                      << KCalendarCore::Journal::journalMimeType();
-    }
-
-    for (int i = 0; i < goodMimeTypes.count(); ++i) {
-        if (collection.contentMimeTypes().contains(goodMimeTypes.at(i))) {
-            return true;
-        }
-    }
-
-    return false;
-}
-
 /**
  * Automatically checks new calendar entries
  */
@@ -156,198 +148,24 @@ protected:
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override
     {
         if (role == Qt::ToolTipRole) {
+#if AKONADI_VERSION < QT_VERSION_CHECK(5, 20, 41)
             const Akonadi::Collection col = CalendarSupport::collectionFromIndex(index);
+#else
+            const Akonadi::Collection col = Akonadi::CollectionUtils::fromIndex(index);
+#endif
             return CalendarSupport::toolTipString(col);
         }
 
         return QSortFilterProxyModel::data(index, role);
-    };
+    }
 };
 
-class KalendarCollectionFilterProxyModel : public Akonadi::CollectionFilterProxyModel
+Q_GLOBAL_STATIC(CalendarManager, calendarManagerGlobalInstance)
+
+CalendarManager *CalendarManager::instance()
 {
-public:
-    explicit KalendarCollectionFilterProxyModel(QObject *parent = nullptr)
-        : Akonadi::CollectionFilterProxyModel(parent)
-    {
-    }
-
-protected:
-    bool lessThan(const QModelIndex &source_left, const QModelIndex &source_right) const override
-    {
-        const auto leftHasChildren = sourceModel()->hasChildren(source_left);
-        const auto rightHasChildren = sourceModel()->hasChildren(source_right);
-        if (leftHasChildren && !rightHasChildren) {
-            return false;
-        } else if (!leftHasChildren && rightHasChildren) {
-            return true;
-        }
-
-        return Akonadi::CollectionFilterProxyModel::lessThan(source_left, source_right);
-    }
-};
-
-/// Despite the name, this handles the presentation of collections including display text and icons, not just colors.
-class ColorProxyModel : public QSortFilterProxyModel
-{
-    Q_OBJECT
-
-public:
-    enum Roles {
-        isResource = Akonadi::EntityTreeModel::UserRole + 1,
-    };
-    Q_ENUM(Roles);
-
-    explicit ColorProxyModel(QObject *parent = nullptr)
-        : QSortFilterProxyModel(parent)
-        , mInitDefaultCalendar(false)
-    {
-        // Needed to read colorattribute of collections for incidence colors
-        Akonadi::AttributeFactory::registerAttribute<Akonadi::CollectionColorAttribute>();
-
-        // Used to get color settings from KOrganizer as fallback
-        const auto korganizerrc = KSharedConfig::openConfig(QStringLiteral("korganizerrc"));
-        const auto skel = new KCoreConfigSkeleton(korganizerrc);
-        mEventViewsPrefs = EventViews::PrefsPtr(new EventViews::Prefs(skel));
-        mEventViewsPrefs->readConfig();
-
-        load();
-    }
-
-    QVariant data(const QModelIndex &index, int role) const override
-    {
-        if (!index.isValid()) {
-            return {};
-        }
-
-        if (role == Qt::DecorationRole) {
-            const Akonadi::Collection collection = CalendarSupport::collectionFromIndex(index);
-
-            if (hasCompatibleMimeTypes(collection)) {
-                if (collection.hasAttribute<Akonadi::EntityDisplayAttribute>()
-                    && !collection.attribute<Akonadi::EntityDisplayAttribute>()->iconName().isEmpty()) {
-                    return collection.attribute<Akonadi::EntityDisplayAttribute>()->iconName();
-                }
-            }
-        } else if (role == Qt::FontRole) {
-            const Akonadi::Collection collection = CalendarSupport::collectionFromIndex(index);
-            if (!collection.contentMimeTypes().isEmpty() && isStandardCalendar(collection.id()) && collection.rights() & Akonadi::Collection::CanCreateItem) {
-                auto font = qvariant_cast<QFont>(QSortFilterProxyModel::data(index, Qt::FontRole));
-                font.setBold(true);
-                if (!mInitDefaultCalendar) {
-                    mInitDefaultCalendar = true;
-                    CalendarSupport::KCalPrefs::instance()->setDefaultCalendarId(collection.id());
-                }
-                return font;
-            }
-        } else if (role == Qt::DisplayRole) {
-            const Akonadi::Collection collection = CalendarSupport::collectionFromIndex(index);
-            const Akonadi::Collection::Id colId = collection.id();
-            const Akonadi::AgentInstance instance = Akonadi::AgentManager::self()->instance(collection.resource());
-
-            if (!instance.isOnline() && !collection.isVirtual()) {
-                return i18nc("@item this is the default calendar", "%1 (Offline)", collection.displayName());
-            }
-            if (colId == CalendarSupport::KCalPrefs::instance()->defaultCalendarId()) {
-                return i18nc("@item this is the default calendar", "%1 (Default)", collection.displayName());
-            }
-        } else if (role == Qt::BackgroundRole) {
-            auto color = getCollectionColor(CalendarSupport::collectionFromIndex(index));
-            // Otherwise QML will get black
-            if (color.isValid()) {
-                return color;
-            } else {
-                return {};
-            }
-        } else if (role == isResource) {
-            return Akonadi::CollectionUtils::isResource(CalendarSupport::collectionFromIndex(index));
-        }
-
-        return QSortFilterProxyModel::data(index, role);
-    }
-
-    Qt::ItemFlags flags(const QModelIndex &index) const override
-    {
-        return Qt::ItemIsSelectable | QSortFilterProxyModel::flags(index);
-    }
-
-    QHash<int, QByteArray> roleNames() const override
-    {
-        QHash<int, QByteArray> roleNames = QSortFilterProxyModel::roleNames();
-        roleNames[Qt::CheckStateRole] = "checkState";
-        roleNames[Qt::BackgroundRole] = "collectionColor";
-        roleNames[isResource] = "isResource";
-        return roleNames;
-    }
-
-    QColor getCollectionColor(Akonadi::Collection collection) const
-    {
-        const QString id = QString::number(collection.id());
-        auto supportsMimeType = collection.contentMimeTypes().contains(QLatin1String("application/x-vnd.akonadi.calendar.event"))
-            || collection.contentMimeTypes().contains(QLatin1String("application/x-vnd.akonadi.calendar.todo"))
-            || collection.contentMimeTypes().contains(QLatin1String("application/x-vnd.akonadi.calendar.journal"));
-        // qDebug() << "Collection id: " << collection.id();
-
-        if (!supportsMimeType) {
-            return {};
-        }
-
-        if (colorCache.contains(id)) {
-            return colorCache[id];
-        }
-
-        if (collection.hasAttribute<Akonadi::CollectionColorAttribute>()) {
-            const auto colorAttr = collection.attribute<Akonadi::CollectionColorAttribute>();
-            if (colorAttr && colorAttr->color().isValid()) {
-                colorCache[id] = colorAttr->color();
-                save();
-                return colorAttr->color();
-            }
-        }
-
-        QColor korgColor = mEventViewsPrefs->resourceColorKnown(id);
-        if (korgColor.isValid()) {
-            colorCache[id] = korgColor;
-            save();
-            return korgColor;
-        }
-
-        QColor color;
-        color.setRgb(QRandomGenerator::global()->bounded(256), QRandomGenerator::global()->bounded(256), QRandomGenerator::global()->bounded(256));
-        colorCache[id] = color;
-        save();
-
-        return color;
-    }
-
-    void load()
-    {
-        KSharedConfig::Ptr config = KSharedConfig::openConfig();
-        KConfigGroup rColorsConfig(config, "Resources Colors");
-        const QStringList colorKeyList = rColorsConfig.keyList();
-
-        for (const QString &key : colorKeyList) {
-            QColor color = rColorsConfig.readEntry(key, QColor("blue"));
-            colorCache[key] = color;
-        }
-    }
-
-    void save() const
-    {
-        KSharedConfig::Ptr config = KSharedConfig::openConfig();
-        KConfigGroup rColorsConfig(config, "Resources Colors");
-        for (auto it = colorCache.constBegin(); it != colorCache.constEnd(); ++it) {
-            rColorsConfig.writeEntry(it.key(), it.value(), KConfigBase::Notify | KConfigBase::Normal);
-        }
-        config->sync();
-    }
-
-    mutable QHash<QString, QColor> colorCache;
-
-private:
-    mutable bool mInitDefaultCalendar;
-    EventViews::PrefsPtr mEventViewsPrefs;
-};
+    return calendarManagerGlobalInstance;
+}
 
 CalendarManager::CalendarManager(QObject *parent)
     : QObject(parent)
@@ -391,6 +209,7 @@ CalendarManager::CalendarManager(QObject *parent)
     m_eventMimeTypeFilterModel = new Akonadi::CollectionFilterProxyModel(this);
     m_eventMimeTypeFilterModel->setSourceModel(collectionFilter);
     m_eventMimeTypeFilterModel->addMimeTypeFilter(QStringLiteral("application/x-vnd.akonadi.calendar.event"));
+
     // text/calendar mimetype includes todo cals
     m_todoMimeTypeFilterModel = new Akonadi::CollectionFilterProxyModel(this);
     m_todoMimeTypeFilterModel->setSourceModel(collectionFilter);
@@ -410,36 +229,16 @@ CalendarManager::CalendarManager(QObject *parent)
     m_todoRightsFilterModel->setAccessRights(Collection::CanCreateItem);
     m_todoRightsFilterModel->setSourceModel(m_todoMimeTypeFilterModel);
 
-    // Use our custom class to order them properly
-    m_selectableCollectionsModel = new KalendarCollectionFilterProxyModel(this);
-    m_selectableCollectionsModel->setSourceModel(m_allCollectionsRightsFilterModel);
-    m_selectableCollectionsModel->addMimeTypeFilter(QStringLiteral("application/x-vnd.akonadi.calendar.event"));
-    m_selectableCollectionsModel->addMimeTypeFilter(QStringLiteral("application/x-vnd.akonadi.calendar.todo"));
-    m_selectableCollectionsModel->setSortCaseSensitivity(Qt::CaseInsensitive);
-    m_selectableCollectionsModel->sort(0, Qt::AscendingOrder);
-
-    m_selectableEventCollectionsModel = new KalendarCollectionFilterProxyModel(this);
-    m_selectableEventCollectionsModel->setSourceModel(m_eventRightsFilterModel);
-    m_selectableEventCollectionsModel->addMimeTypeFilter(QStringLiteral("application/x-vnd.akonadi.calendar.event"));
-    m_selectableEventCollectionsModel->setSortCaseSensitivity(Qt::CaseInsensitive);
-    m_selectableEventCollectionsModel->sort(0, Qt::AscendingOrder);
-
-    m_selectableTodoCollectionsModel = new KalendarCollectionFilterProxyModel(this);
-    m_selectableTodoCollectionsModel->setSourceModel(m_todoRightsFilterModel);
-    m_selectableTodoCollectionsModel->addMimeTypeFilter(QStringLiteral("application/x-vnd.akonadi.calendar.todo"));
-    m_selectableTodoCollectionsModel->setSortCaseSensitivity(Qt::CaseInsensitive);
-    m_selectableTodoCollectionsModel->sort(0, Qt::AscendingOrder);
-
     // Model for todo via collection picker
-    m_todoViewCollectionModel = new KalendarCollectionFilterProxyModel(this);
+    m_todoViewCollectionModel = new SortedCollectionProxModel(this);
     m_todoViewCollectionModel->setSourceModel(collectionFilter);
     m_todoViewCollectionModel->addMimeTypeFilter(QStringLiteral("application/x-vnd.akonadi.calendar.todo"));
     m_todoViewCollectionModel->setExcludeVirtualCollections(true);
     m_todoViewCollectionModel->setSortCaseSensitivity(Qt::CaseInsensitive);
     m_todoViewCollectionModel->sort(0, Qt::AscendingOrder);
 
-    // Model for the sidebar
-    m_viewCollectionModel = new KalendarCollectionFilterProxyModel(this);
+    // Model for the mainDrawer
+    m_viewCollectionModel = new SortedCollectionProxModel(this);
     m_viewCollectionModel->setSourceModel(collectionFilter);
     m_viewCollectionModel->addMimeTypeFilter(QStringLiteral("application/x-vnd.akonadi.calendar.event"));
     m_viewCollectionModel->addMimeTypeFilter(QStringLiteral("application/x-vnd.akonadi.calendar.todo"));
@@ -454,14 +253,20 @@ CalendarManager::CalendarManager(QObject *parent)
     auto refreshColors = [=]() {
         for (auto i = 0; i < m_flatCollectionTreeModel->rowCount(); i++) {
             auto idx = m_flatCollectionTreeModel->index(i, 0, {});
+#if AKONADI_VERSION < QT_VERSION_CHECK(5, 20, 41)
             colorProxy->getCollectionColor(CalendarSupport::collectionFromIndex(idx));
+#else
+            colorProxy->getCollectionColor(Akonadi::CollectionUtils::fromIndex(idx));
+#endif
         }
     };
     connect(m_flatCollectionTreeModel, &QSortFilterProxyModel::rowsInserted, this, refreshColors);
 
     KConfigGroup rColorsConfig(config, "Resources Colors");
     m_colorWatcher = KConfigWatcher::create(config);
-    QObject::connect(m_colorWatcher.data(), &KConfigWatcher::configChanged, this, &CalendarManager::collectionColorsChanged);
+    connect(m_colorWatcher.data(), &KConfigWatcher::configChanged, this, &CalendarManager::collectionColorsChanged);
+
+    connect(m_calendar.data(), &Akonadi::ETMCalendar::calendarChanged, this, &CalendarManager::calendarChanged);
 }
 
 CalendarManager::~CalendarManager()
@@ -484,7 +289,6 @@ void CalendarManager::save()
 
 void CalendarManager::delayedInit()
 {
-    Q_EMIT entityTreeModelChanged();
     Q_EMIT loadingChanged();
 }
 
@@ -562,21 +366,6 @@ Akonadi::CollectionFilterProxyModel *CalendarManager::allCalendars()
     return m_allCalendars;
 }
 
-Akonadi::CollectionFilterProxyModel *CalendarManager::selectableCalendars() const
-{
-    return m_selectableCollectionsModel;
-}
-
-Akonadi::CollectionFilterProxyModel *CalendarManager::selectableEventCalendars() const
-{
-    return m_selectableEventCollectionsModel;
-}
-
-Akonadi::CollectionFilterProxyModel *CalendarManager::selectableTodoCalendars() const
-{
-    return m_selectableTodoCollectionsModel;
-}
-
 qint64 CalendarManager::defaultCalendarId(IncidenceWrapper *incidenceWrapper)
 {
     // Checks if default collection accepts this type of incidence
@@ -604,33 +393,6 @@ qint64 CalendarManager::defaultCalendarId(IncidenceWrapper *incidenceWrapper)
     return -1;
 }
 
-int CalendarManager::getCalendarSelectableIndex(IncidenceWrapper *incidenceWrapper)
-{
-    auto model = new KDescendantsProxyModel;
-
-    switch (incidenceWrapper->incidencePtr()->type()) {
-    default:
-    case (KCalendarCore::IncidenceBase::TypeEvent): {
-        model->setSourceModel(m_selectableEventCollectionsModel);
-        break;
-    }
-    case (KCalendarCore::IncidenceBase::TypeTodo): {
-        model->setSourceModel(m_selectableTodoCollectionsModel);
-        break;
-    }
-    }
-
-    for (int i = 0; i < model->rowCount(); i++) {
-        QModelIndex idx = model->index(i, 0);
-        QVariant data = idx.data(Akonadi::EntityTreeModel::Roles::CollectionIdRole);
-
-        if (data == incidenceWrapper->collectionId())
-            return i;
-    }
-
-    return 0;
-}
-
 QVariant CalendarManager::getIncidenceSubclassed(KCalendarCore::Incidence::Ptr incidencePtr)
 {
     switch (incidencePtr->type()) {
@@ -659,11 +421,21 @@ QVariantMap CalendarManager::undoRedoData()
     };
 }
 
-Akonadi::Item CalendarManager::incidenceItem(KCalendarCore::Incidence::Ptr incidence)
+Akonadi::Item CalendarManager::incidenceItem(KCalendarCore::Incidence::Ptr incidence) const
 {
     return m_calendar->item(incidence);
 }
 
+Akonadi::Item CalendarManager::incidenceItem(const QString &uid) const
+{
+    return incidenceItem(m_calendar->incidence(uid));
+}
+
+KCalendarCore::Incidence::List CalendarManager::childIncidences(const QString &uid) const
+{
+    return m_calendar->childIncidences(uid);
+}
+
 void CalendarManager::addIncidence(IncidenceWrapper *incidenceWrapper)
 {
     Akonadi::Collection collection(incidenceWrapper->collectionId());
@@ -911,7 +683,7 @@ QVariantMap CalendarManager::getCollectionDetails(QVariant collectionId)
     return collectionDetails;
 }
 
-void CalendarManager::setCollectionColor(qint64 collectionId, QColor color)
+void CalendarManager::setCollectionColor(qint64 collectionId, const QColor &color)
 {
     auto collection = m_calendar->collection(collectionId);
     auto colorAttr = collection.attribute<Akonadi::CollectionColorAttribute>(Akonadi::Collection::AddIfMissing);
@@ -981,6 +753,23 @@ void CalendarManager::editCollection(qint64 collectionId)
     dlg->show();
 }
 
+void CalendarManager::toggleCollection(qint64 collectionId)
+{
+    const auto matches = m_calendar->checkableProxyModel()->match(m_calendar->checkableProxyModel()->index(0, 0),
+                                                                  Akonadi::EntityTreeModel::CollectionIdRole,
+                                                                  collectionId,
+                                                                  1,
+                                                                  Qt::MatchExactly | Qt::MatchWrap | Qt::MatchRecursive);
+    if (matches.count() > 0) {
+        const auto collectionIndex = matches.first();
+        const auto collectionChecked = collectionIndex.data(Qt::CheckStateRole).toInt() == Qt::Checked;
+        const auto checkStateToSet = collectionChecked ? Qt::Unchecked : Qt::Checked;
+        m_calendar->checkableProxyModel()->setData(collectionIndex, checkStateToSet, Qt::CheckStateRole);
+    }
+}
+
+#ifndef UNITY_CMAKE_SUPPORT
 Q_DECLARE_METATYPE(KCalendarCore::Incidence::Ptr)
+#endif
 
 #include "calendarmanager.moc"
