@@ -25,6 +25,18 @@ IncidenceWrapper::IncidenceWrapper(QObject *parent)
         m_attachmentsModel.setIncidencePtr(incidencePtr);
     });
 
+    // While generally we know of the relationship an incidence has regarding its parent,
+    // from the POV of an incidence, we have no idea of its relationship to its children.
+    // This is a limitation of KCalendarCore, which only supports one type of relationship
+    // type per incidence and throughout the PIM infrastructure it is always the 'parent'
+    // relationship that is used.
+
+    // We therefore need to rely on the ETMCalendar for this information. Since the ETMCalendar
+    // does not provide us with any specific information about the incidences changed when it
+    // updates, we unfortunately have to this the coarse way and just update everything when
+    // things change.
+    connect(CalendarManager::instance(), &CalendarManager::calendarChanged, this, &IncidenceWrapper::resetChildIncidences);
+
     Akonadi::ItemFetchScope scope;
     scope.fetchFullPayload();
     scope.fetchAllAttributes();
@@ -35,7 +47,10 @@ IncidenceWrapper::IncidenceWrapper(QObject *parent)
     setNewEvent();
 }
 
-IncidenceWrapper::~IncidenceWrapper() = default;
+IncidenceWrapper::~IncidenceWrapper()
+{
+    cleanupChildIncidences();
+}
 
 void IncidenceWrapper::notifyDataChanged()
 {
@@ -43,6 +58,9 @@ void IncidenceWrapper::notifyDataChanged()
     Q_EMIT incidenceTypeStrChanged();
     Q_EMIT incidenceIconNameChanged();
     Q_EMIT collectionIdChanged();
+    Q_EMIT parentChanged();
+    Q_EMIT parentIncidenceChanged();
+    Q_EMIT childIncidencesChanged();
     Q_EMIT summaryChanged();
     Q_EMIT categoriesChanged();
     Q_EMIT descriptionChanged();
@@ -56,6 +74,8 @@ void IncidenceWrapper::notifyDataChanged()
     Q_EMIT timeZoneChanged();
     Q_EMIT startTimeZoneUTCOffsetMinsChanged();
     Q_EMIT endTimeZoneUTCOffsetMinsChanged();
+    Q_EMIT durationChanged();
+    Q_EMIT durationDisplayStringChanged();
     Q_EMIT allDayChanged();
     Q_EMIT priorityChanged();
     Q_EMIT remindersModelChanged();
@@ -79,6 +99,7 @@ void IncidenceWrapper::setIncidenceItem(const Akonadi::Item &incidenceItem)
     if (incidenceItem.hasPayload<KCalendarCore::Incidence::Ptr>()) {
         setItem(incidenceItem);
         setIncidencePtr(incidenceItem.payload<KCalendarCore::Incidence::Ptr>());
+
         Q_EMIT incidenceItemChanged();
         Q_EMIT collectionIdChanged();
     } else {
@@ -115,7 +136,7 @@ int IncidenceWrapper::incidenceType() const
 
 QString IncidenceWrapper::incidenceTypeStr() const
 {
-    return m_incidence->type() == KCalendarCore::Incidence::TypeTodo ? i18n("Task") : i18n(m_incidence->typeStr());
+    return m_incidence->type() == KCalendarCore::Incidence::TypeTodo ? i18n("Task") : i18n(m_incidence->typeStr().constData());
 }
 
 QString IncidenceWrapper::incidenceIconName() const
@@ -147,9 +168,22 @@ QString IncidenceWrapper::parent() const
 void IncidenceWrapper::setParent(QString parent)
 {
     m_incidence->setRelatedTo(parent);
+    updateParentIncidence();
     Q_EMIT parentChanged();
 }
 
+IncidenceWrapper *IncidenceWrapper::parentIncidence()
+{
+    updateParentIncidence();
+    return m_parentIncidence.data();
+}
+
+QVariantList IncidenceWrapper::childIncidences()
+{
+    resetChildIncidences();
+    return m_childIncidences;
+}
+
 QString IncidenceWrapper::summary() const
 {
     return m_incidence->summary();
@@ -249,6 +283,8 @@ void IncidenceWrapper::setIncidenceStart(const QDateTime &incidenceStart, bool r
     Q_EMIT incidenceStartChanged();
     Q_EMIT incidenceStartDateDisplayChanged();
     Q_EMIT incidenceStartTimeDisplayChanged();
+    Q_EMIT durationChanged();
+    Q_EMIT durationDisplayStringChanged();
 }
 
 void IncidenceWrapper::setIncidenceStartDate(int day, int month, int year)
@@ -320,6 +356,8 @@ void IncidenceWrapper::setIncidenceEnd(const QDateTime &incidenceEnd, bool respe
     Q_EMIT incidenceEndChanged();
     Q_EMIT incidenceEndDateDisplayChanged();
     Q_EMIT incidenceEndTimeDisplayChanged();
+    Q_EMIT durationChanged();
+    Q_EMIT durationDisplayStringChanged();
 }
 
 void IncidenceWrapper::setIncidenceEndDate(int day, int month, int year)
@@ -403,6 +441,21 @@ int IncidenceWrapper::endTimeZoneUTCOffsetMins()
     return QTimeZone(timeZone()).offsetFromUtc(incidenceEnd());
 }
 
+KCalendarCore::Duration IncidenceWrapper::duration() const
+{
+    return m_incidence->duration();
+}
+
+QString IncidenceWrapper::durationDisplayString() const
+{
+    const auto dur = duration();
+    if (dur.asSeconds() == 0) {
+        return QString();
+    } else {
+        return m_format.formatSpelloutDuration(dur.asSeconds() * 1000);
+    }
+}
+
 bool IncidenceWrapper::allDay() const
 {
     return m_incidence->allDay();
@@ -648,8 +701,10 @@ void IncidenceWrapper::setTodoPercentComplete(int todoPercentComplete)
 
 void IncidenceWrapper::triggerEditMode() // You edit a clone so that the original ptr isn't messed with
 {
+    auto itemToEdit = item();
     KCalendarCore::Incidence::Ptr clonedPtr(m_incidence->clone());
-    setIncidencePtr(clonedPtr);
+    itemToEdit.setPayload<KCalendarCore::Incidence::Ptr>(clonedPtr);
+    setIncidenceItem(itemToEdit);
 }
 
 static int nearestQuarterHour(int secsSinceEpoch)
@@ -666,19 +721,82 @@ void IncidenceWrapper::setNewEvent()
     event->setDtStart(start);
     event->setDtEnd(start.addSecs(60 * 60));
 
-    Akonadi::Item incidenceItem;
-    incidenceItem.setPayload<KCalendarCore::Event::Ptr>(event);
-    setIncidenceItem(incidenceItem);
+    KCalendarCore::Alarm::Ptr alarm(new KCalendarCore::Alarm(event.get()));
+    alarm->setEnabled(true);
+    alarm->setType(KCalendarCore::Alarm::Display);
+    alarm->setStartOffset(-1 * 15 * 60); // 15 minutes
+
+    event->addAlarm(alarm);
+
+    setNewIncidence(event);
 }
 
 void IncidenceWrapper::setNewTodo()
 {
     auto todo = KCalendarCore::Todo::Ptr(new KCalendarCore::Todo);
+    setNewIncidence(todo);
+}
+
+void IncidenceWrapper::setNewIncidence(KCalendarCore::Incidence::Ptr incidence)
+{
     Akonadi::Item incidenceItem;
-    incidenceItem.setPayload<KCalendarCore::Todo::Ptr>(todo);
+    incidenceItem.setPayload<KCalendarCore::Incidence::Ptr>(incidence);
     setIncidenceItem(incidenceItem);
 }
 
+// We need to be careful when we call updateParentIncidence and resetChildIncidences.
+// For instance, we always call them on-demand based on access to the properties and not
+// upon object construction on upon setting the incidence pointer.
+
+// Calling them both recursively down a family tree can cause a cascade of infinite
+// new IncidenceWrappers being created. Say we create a new incidence wrapper here and
+// call this new incidence's updateParentIncidence and resetChildIncidences, creating
+// a new child wrapper, creating more wrappers there, and so on.
+
+void IncidenceWrapper::updateParentIncidence()
+{
+    if (!m_incidence) {
+        return;
+    }
+
+    if (!parent().isEmpty() && (!m_parentIncidence || m_parentIncidence->uid() != parent())) {
+        m_parentIncidence.reset(new IncidenceWrapper);
+        m_parentIncidence->setIncidenceItem(CalendarManager::instance()->incidenceItem(parent()));
+        Q_EMIT parentIncidenceChanged();
+    }
+}
+
+void IncidenceWrapper::resetChildIncidences()
+{
+    cleanupChildIncidences();
+
+    if (!m_incidence) {
+        return;
+    }
+
+    const auto incidences = CalendarManager::instance()->childIncidences(uid());
+    QVariantList wrappedIncidences;
+
+    for (const auto &incidence : incidences) {
+        const auto wrappedIncidence = new IncidenceWrapper;
+        wrappedIncidence->setIncidenceItem(CalendarManager::instance()->incidenceItem(incidence));
+        wrappedIncidences.append(QVariant::fromValue(wrappedIncidence));
+    }
+
+    m_childIncidences = wrappedIncidences;
+    Q_EMIT childIncidencesChanged();
+}
+
+void IncidenceWrapper::cleanupChildIncidences()
+{
+    while (!m_childIncidences.isEmpty()) {
+        const auto incidence = m_childIncidences.takeFirst();
+        const auto incidencePtr = incidence.value<IncidenceWrapper *>();
+
+        delete incidencePtr;
+    }
+}
+
 void IncidenceWrapper::addAlarms(KCalendarCore::Alarm::List alarms)
 {
     for (int i = 0; i < alarms.size(); i++) {
@@ -734,9 +852,16 @@ void IncidenceWrapper::itemChanged(const Akonadi::Item &item)
 {
     if (item.hasPayload<KCalendarCore::Incidence::Ptr>()) {
         qCDebug(KALENDAR_LOG) << item.payload<KCalendarCore::Incidence::Ptr>()->summary() << item.parentCollection().id();
-        setIncidencePtr(item.payload<KCalendarCore::Incidence::Ptr>());
+        setIncidenceItem(item);
     }
 }
+
+// TODO remove with 22.08, won't be needed anymore
+void IncidenceWrapper::setCollection(const Akonadi::Collection &collection)
+{
+    setCollectionId(collection.id());
+}
+
 #ifndef UNITY_CMAKE_SUPPORT
-Q_DECLARE_METATYPE(KCalendarCore::Incidence::Ptr);
+Q_DECLARE_METATYPE(KCalendarCore::Incidence::Ptr)
 #endif
