@@ -12,9 +12,9 @@ class TimeZoneListModel : public QAbstractListModel
 
 public:
     enum Roles { IdRole = Qt::UserRole + 1 };
-    Q_ENUM(Roles);
+    Q_ENUM(Roles)
 
-    TimeZoneListModel(QObject *parent = nullptr);
+    explicit TimeZoneListModel(QObject *parent = nullptr);
     ~TimeZoneListModel() override = default;
 
     QVariant data(const QModelIndex &idx, int role) const override;
